#!/usr/bin/env python

import argparse
import jobwatch
from jobwatch import (FileWatch, JobWatch, DbWatch,
                      make_html_report, copy_errs,
                      set_report_attrs)


# Ska-specific watchers
class SkaWebWatch(FileWatch):
    def __init__(self, task, maxage, basename,
                 filename='/proj/sot/ska/www/ASPECT/{task}/{basename}'):
        self.basename = basename
        super(SkaWebWatch, self).__init__(task, maxage, filename)


class SkaJobWatch(JobWatch):
    def __init__(self, task, maxage=1, errors=jobwatch.ERRORS, requires=(),
                 logdir='logs', logtask=None,
                 filename='/proj/sot/ska/data/{task}/' \
                         '{logdir}/daily.0/{logtask}.log'):
        self.type = 'Log'
        self.task = task
        self.logtask = logtask or task
        self.logdir = logdir
        super(SkaJobWatch, self).__init__(task, filename, errors=errors,
                                          requires=requires, maxage=maxage)


class SkaDbWatch(DbWatch):
    def __init__(self, task, maxage=1, table=None, timekey='tstart'):
        super(SkaDbWatch, self).__init__(
            task, maxage=maxage, table=table, timekey=timekey,
            query='SELECT MAX({timekey}) AS maxtime FROM {table}',
            dbi='sybase', server='sybase', user='aca_read', database='aca')


parser = argparse.ArgumentParser(description='Ska processing monitor')
parser.add_argument('--date-now',
                    help='Processing date')
parser.add_argument('--rootdir',
                    default='.',
                    help='Output root directory')
parser.add_argument('--email',
                    action='store_true',
                    help='Send email report')
parser.add_argument('--loud',
                    action='store_true',
                    help='Send email report')
parser.add_argument('--max-age',
                    type=int,
                    default=30,
                    help='Maximum age of watch reports')
args = parser.parse_args()

jobwatch.LOUD = args.loud

# Customized errors and paths
py_errs = set(('error', 'warn', 'fail', 'fatal', 'exception', 'traceback'))
perl_errs = set(('uninitialized value',
                 '(?<!Program caused arithmetic )error',
                 'warn', 'fatal', 'fail', 'undefined value'))
arc_errs = copy_errs(perl_errs, ['warn'], ['warning(?!:\s+\d+\s)'])
nmass_errs = copy_errs(py_errs, ('warn', 'fail'),
                       ('warn(?!ing: imaging routines will not be available)',
                        'fail(?!ed to import sherpa)'))
psmc_errs = copy_errs(py_errs.union(perl_errs), ['traceback'],
                      ["traceback(?!': True)"])
telem_archive_errs = copy_errs(py_errs, ['fail'],
                               ['(?<!...)fail(?!...)'])
perigee_errs = copy_errs(py_errs, ['warn'],
                         ['warn(?!ing: limit exceeded, dac of)'])
astromon_errs = ('uninitialized value', 'warn', 'fatal', 'fail',
                 'undefined value',
                 'ERROR(?!: Data::ParseTable: FITS ' +
                 'files cannot be passed as arrays)')
engarchive_errs = copy_errs(py_errs, ['fail'],
                             ['(?<!5OHW)FAIL(?!MODE)'])
perigee_errs = copy_errs(py_errs, ['warn'],
                         ['warning(?!: Limit Exceeded. dac of)'])
jean_db = '/proj/sot/ska/data/database/Logs/daily.0/{task}.log'
star_stat = '/proj/sot/ska/data/star_stat_db/Logs/daily.0/{task}.log'

jws = []
jws.extend([
    SkaJobWatch('aca_bgd_mon', 40, errors=perl_errs,
                requires=('Copying plots and log file '
                          'to /proj/sot/ska/www/ASPECT',)),
    SkaJobWatch('arc', 2, errors=arc_errs, logdir='Logs'),
    SkaJobWatch('astromon', 8, errors=astromon_errs),
    SkaJobWatch('dsn_summary', 2, errors=perl_errs),
    SkaJobWatch('eng_archive', 2, errors=engarchive_errs),
    SkaJobWatch('fid_drift_mon', 2, errors=py_errs.union(perl_errs)),
    SkaJobWatch('star_stats', 2, filename=star_stat),
    SkaJobWatch('timelines', 2, logtask='timelines_cmd_states', logdir='Logs'),
    SkaJobWatch('taco', 8),
    SkaJobWatch('acq_database', 2, filename=jean_db),
    SkaJobWatch('guide_database', 2, filename=jean_db),
    SkaJobWatch('guide_stat_db', 2, filename=jean_db),
    SkaJobWatch('load_database', 2, filename=jean_db),
    SkaJobWatch('obsid_load_database', 2, filename=jean_db),
    SkaJobWatch('star_database', 2, filename=jean_db),
    SkaJobWatch('starcheck_database', 2, filename=jean_db),
    SkaJobWatch('vv_database', 2, filename=jean_db),
    SkaJobWatch('nmass', 8, errors=nmass_errs, logtask='trend_nmass'),
    SkaJobWatch('psmc', 2, logtask='psmc_daily_check', errors=psmc_errs),
    SkaJobWatch('scs107', 2, logdir='Logs', logtask='scs107_check'),
    SkaJobWatch('telem_archive', 2, errors=telem_archive_errs),
    SkaJobWatch('perigee_health_plots', 2, logdir='Logs',
                errors=perigee_errs),
    ])

jws.extend([
    SkaWebWatch('acq_stat_reports', 10, 'index.html'),
    SkaWebWatch('arc', 1, 'index.html'),
    SkaWebWatch('arc', 1, 'chandra.snapshot'),
    SkaWebWatch('arc', 1, 'hrc_shield.png'),
    SkaWebWatch('arc', 1, 'GOES_xray.gif'),
    SkaWebWatch('arc', 1, 'GOES_5min.gif'),
    SkaWebWatch('arc', 1, 'solar_wind.gif'),
    SkaWebWatch('arc', 1, 'solar_flare_monitor.png'),
    SkaWebWatch('arc', 1, 'ACE_5min.gif'),
    SkaWebWatch('celmon', 30, 'offsets-ACIS-S-hist.gif'),
    FileWatch('dsn_summary', 1,
              '/proj/sot/ska/data/dsn_summary/dsn_summary.dat'),
    SkaWebWatch('gui_stat_reports', 10, 'index.html'),
    SkaWebWatch('fid_drift', 2, 'drift_acis_s.png'),
    SkaWebWatch('obc_rate_noise', 50, 'trending/pitch_hist_recent.png'),
    SkaWebWatch('perigee_health_plots', 5, 'index.html'),
    ])

jws.extend([
    SkaDbWatch('acq_stats_data', 4),
    SkaDbWatch('aiprops', 4),
    SkaDbWatch('cmds', -1, timekey='date'),
    SkaDbWatch('cmd_states', -1, timekey='datestart'),
    SkaDbWatch('load_segments', -1, timekey='datestop'),
    SkaDbWatch('obspar', 4),
    SkaDbWatch('starcheck_obs', 4, timekey='mp_starcat_time'),
    SkaDbWatch('trak_stats_data', 4, timekey='kalman_tstart'),
    ])

set_report_attrs(jws)
index_html = make_html_report(jws, args.rootdir, args.date_now)
recipients = ['aca@head.cfa.harvard.edu']

if args.email:
    jobwatch.sendmail(recipients, index_html, args.date_now)

jobwatch.remove_old_reports(args.rootdir, args.date_now, args.max_age)
